
#define AliasesSet	0x5
#define AliasesDelete	0x1
#define AliasesDeletePlural	0x2
#define AliasesKeepDelete	0x3
#define AliasesKeepDeletePlural	0x4
#define AliasesDeleteAll	0x5
#define AliasesSelect	0x7
#define AliasesSingle	0x8
#define AliasesPlural	0x9
#define AliasesRMenuLn1	0xa
#define AliasesRMenuLn2	0xb
#define AliasesRMenuLn3	0xc
#define AliasesRMenuLn4	0xd
#define AliasesMenuLn1	0xe
#define AliasesMenuLn2	0xf
#define AliasesMenuLn3	0x10
#define AliasesSort	0x11
#define AliasesGroup	0x12
#define AliasesPerson	0x13
#define AliasesSystemFlag	0x14
#define AliasesRevAliasAbr	0x28
#define AliasesRevAliasName	0x29
#define AliasesRevFullAbr	0x2a
#define AliasesRevFullName	0x2b
#define AliasesRevTextAbr	0x2c
#define AliasesRevTextFile	0x2d
#define AliasesAliasAbr	0x2e
#define AliasesAliasName	0x2f
#define AliasesFullAbr	0x30
#define AliasesFullName	0x31
#define AliasesTextAbr	0x32
#define AliasesTextFile	0x33
#define AliasesEnterAliasName	0x34
#define AliasesAlreadyGroup	0x36
#define AliasesAlreadyAlias	0x37
#define AliasesAcceptNew	0x38
#define AliasesSuperceed	0x39
#define AliasesEnterLastName	0x3a
#define AliasesEnterAddress	0x3b
#define AliasesNoAddressSpec	0x3c
#define AliasesNoMessage	0x3d
#define AliasesCurrentMessage	0x3e
#define AliasesSystemAlias	0x3f
#define AliasesCouldntOpenAdd	0x40
#define AliasesCouldntWrite	0x41
#define AliasesCouldntOpenDelete	0x42
#define AliasesCouldntOpenTemp	0x43
#define AliasesCouldntWriteTemp	0x44
#define AliasesCouldntRenameTemp	0x45
#define AliasesReplaceExisting	0x46
#define AliasesResync	0x49
#define AliasesAddCurrent	0x4a
#define AliasesMail	0x4c
#define AliasesAddNew	0x4d
#define AliasesAddReturn	0x4e
#define AliasesReplaceCurrent	0x4f
#define AliasesNoneToReplace	0x50
#define AliasesNotInstalled	0x54
#define AliasesGroupAlias	0x55
#define AliasesAliasedAddress	0x56
#define AliasesFullyExpanded	0x57
#define AliasesAliasedFull	0x58
#define AliasesPressReturn	0x59
#define AliasesNotFound	0x5a
#define AliasesUpdating	0x5b
#define AliasesReReading	0x5c
#define AliasesUpdatedOK	0x5d
#define AliasesShortKey	0x5f
#define AliasesLongKey	0x60
#define AliasesKeyMenu	0x61
#define AliasesHelpDollar	0x62
#define AliasesHelpv	0x63
#define AliasesHelpa	0x64
#define AliasesHelpd	0x65
#define AliasesHelpCtrlD	0x66
#define AliasesHelpf	0x67
#define AliasesHelpl	0x68
#define AliasesHelpCtrlL	0x69
#define AliasesHelpm	0x6a
#define AliasesHelpn	0x6b
#define AliasesHelpi	0x6c
#define AliasesHelpQ	0x6d
#define AliasesHelpt	0x6f
#define AliasesHelpCtrlT	0x70
#define AliasesHelpu	0x71
#define AliasesHelpCtrlU	0x72
#define AliasesHelpX	0x73
#define AliasesHelpNoHelp	0x74
#define AliasesErrorMemory	0x75
#define Aliasesitem	0x76
#define Aliasesitems	0x77
#define AliasesItem	0x78
#define AliasesItems	0x79
#define AliasesPrompt	0x7a
#define AliasesLimitMessagesSelected	0x7c
#define AliasesLimitMessageSelected	0x7d
#define AliasesLimitNoMessagesSelected	0x7e
#define AliasesEnterLastSelected	0x7f
#define AliasesEnterSelected	0x80
#define AliasesHelpSlash	0x81
#define AliasesNoneToView	0x82
#define AliasesNoneToMail	0x83
#define AliasesEnterFirstName	0x84
#define AliasesEnterComment	0x85
#define AliasesAddressAs	0x86
#define AliasesAddressTo	0x87
#define AliasesOneTagged	0x88
#define AliasesManyTagged	0x89
#define AliasesCreateGroup	0x8a
#define AliasesHelpe	0x8b
#define AliasesEdit	0x8c
#define AliasesHelpc	0x8e
#define AliasesHelpT	0x8f
#define AliasesModified	0x90
