$set 7 #Attach
$ this catalog generated automatically by "getcatstrs" - DO NOT EDIT!
$ #MainMessage
1	[main message]
$ #TooManyAttachments
2	Too many attachments for menu to handle!
$ #ScreenTitle
3	Message Attachments Screen
$ #ScreenPage
4	[page %d/%d]
$ #CurrTitle
5	---------- Current Attachment ----------
$ #CurrFileName
6	F)ile Name
$ #CurrContType
7	C)ontent Type
$ #CurrContEncoding
8	Content E)ncoding
$ #CurrContDescription
9	Content De(s)cription
$ #CurrContDisposition
10	Content Dis(p)osition
$ #MainInstrNorm
11	Use "jk+-" to move; "?" = help; a)dd
$ #MainInstrDummy1
12	Make selection:  j/k = down/up, +/- = down/up page, or enter selection num.
$ #MainInstrDummy2
13	Then a)dd after or d
$quote "
$ #MainPrompt
14	"Command: "
$quote
$ #Attachment
15	attachment
$ #NoAttachmentThere
16	There isn't an attachment there!
$ #SelectionNotChanged
17	Selection not changed.
$ #NoRoom
18	Sorry - no room for any more attachments.
$ #NewAttachment
19	[new attachment]
$ #AttachmentNotAdded
20	Attachment not added.
$ #AttachmentHasBeenAdded
21	Attachment has been added to this message.
$ #YouCantDelete
22	Hey!  You can't delete that!
$ #ReallyDelete
23	Really delete attachment?
$ #AttachmentNotDeleted
24	Attachment not deleted.
$ #AttachmentHasBeenDeleted
25	Attachment has been deleted from this message.
$ #YouCantChange
26	Hey!  You can't change that!
$ #NotChanged
27	Attachment not changed.
$ #AttachmentFileChanged
28	Attachment file has been changed.
$ #AttachmentContTypeChanged
29	Attachment content type has been changed.
$ #AttachmentContEncodingChanged
30	Attachment content encoding has been changed.
$ #AttachmentContDescriptionChanged
31	Attachment content description has been changed.
$ #AttachmentContDispositionChanged
32	Attachment content disposition has been changed.
$ #YouCantTag
33	Hey!  You can't tag that!
$ #AttachmentHasBeenTagged
34	Current attachment has been tagged.
$ #TagHasBeenRemoved
35	Tag has been removed from current attachment.
$ #TagsAlreadyClear
36	All tags already are cleared.
$ #ReallyClearTags
37	Really clear all tags?
$ #TagsNotChanged
38	Tags not changed.
$ #TagsHaveBeenCleared
39	All tags have been cleared.
$ #MustTagAttachments
40	You must tag the attachments you want to move.
$ #MoveTaggedAttachments
41	Move tagged attachments after selection?
$ #MoveTaggedAttachment
42	Move tagged attachment after selection?
$ #AttachmentsNotMoved
43	Attachments not moved.
$ #AttachmentNotMoved
44	Attachment not moved.
$ #AttachmentsHaveBeenMoved
45	Attachments have been moved.
$ #AttachmentHaveBeenMoved
46	Attachment has been moved.
$ #ChgFileInstrNorm
47	Enter attachment filename.  "~", "=", and patterns ok, CTRL/D to abort.
$ #ChgFileInstrDummy1
48	Enter name of file to attach to your mail message, or CTRL/D to abort entry.
$ #ChgFileInstrDummy2
49	Say "." to select a file in the current directory.
$ #ChgTypeInstrNorm
50	Enter attachment content type, CTRL/D to abort.
$ #ChgTypeInstrDummy1
51	Enter the "content type" of this file, or CTRL/D to abort entry.
$ #ChgTypeInstrDummy2
52	You DO know what a Content-Type is...don't you???
$ #ChgTypeEnterType
53	Please enter a content type.
$ #ChgEncodingInstrNorm
54	Enter attachment content encoding, CTRL/D to abort.
$ #ChgEncodingInstrDummy1
55	Enter the "content encoding" of this file, or CTRL/D to abort entry.
$ #ChgEncodingInstrDummy2
56	Common values are "7bit", "8bit", "base64", and "x-uuencode".
$ #ChgDescripInstrNorm
57	Enter attachment content description, CTRL/D to abort.
$ #ChgDescripInstrDummy1
58	Enter the "content description", or CTRL/D to abort entry.
$ #ChgDescripInstrDummy2
59	This is just a brief comment to tell the recipient about the attachment.
$ #ChgDispositionInstrNorm
60	Enter suggested filename (empty OK), CTRL/D to abort.
$ #ChgDispositionInstrDummy1
61	Enter a suggested filename under which the recipient might want to save
$ #ChgDispositionInstrDummy2
62	this attachment (empty filename is OK), or CTRL/D to abort entry.
